package com.sun.messaging.jmq.jmsserver.data;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.sun.messaging.jmq.io.Packet;
import com.sun.messaging.jmq.io.SysMessageID;
import com.sun.messaging.jmq.jmsserver.Globals;
import com.sun.messaging.jmq.jmsserver.core.BrokerAddress;
import com.sun.messaging.jmq.jmsserver.core.ConsumerUID;
import com.sun.messaging.jmq.jmsserver.core.Destination;
import com.sun.messaging.jmq.jmsserver.core.DestinationUID;
import com.sun.messaging.jmq.jmsserver.core.PacketReference;
import com.sun.messaging.jmq.jmsserver.persist.Store;
import com.sun.messaging.jmq.jmsserver.persist.TransactionInfo;
import com.sun.messaging.jmq.jmsserver.resources.BrokerResources;
import com.sun.messaging.jmq.jmsserver.util.BrokerException;
import com.sun.messaging.jmq.util.log.Logger;

public class TransactionListLoader {

	Logger logger = Globals.getLogger();
	Store store = null; // persistence access
	TransactionList transactionList;

	// initial size of txn log file
	public TransactionListLoader(Store store, TransactionList transactionList) {
		this.store = store;
		this.transactionList = transactionList;
	}

	// txn log

	public void loadTransactions() throws BrokerException, IOException {
		logger.log(Logger.INFO, BrokerResources.I_PROCESSING_TRANS);
		// This method is called after the message store has been loaded
		// and after the transaction log has been replayed.

		// The state of the system should be :
		// 1) All committed transactions since the last checkpoint should
		// have been replayed and persisted to the message store.
		// 2) There should be no incomplete transactions
		// (a partial write of a transaction will have been discarded).
		// 3) The message store will contain a set of prepared transactions
		// either from XA or from clustered/remote transactions.
		loadLocalTransactions();
		loadClusterTransactions();
		loadRemoteTransactions();

	}

	public void loadLocalTransactions() throws BrokerException, IOException {

		List<BaseTransaction> incompleteTxns = store
				.getIncompleteTransactions(BaseTransaction.LOCAL_TRANSACTION_TYPE);

		String msg = " loading " + incompleteTxns.size()
				+ " incomplete Local transactions:  ";
		logger.log(Logger.DEBUG, msg);

		Iterator<BaseTransaction> iter = incompleteTxns.iterator();
		while (iter.hasNext()) {
			BaseTransaction baseTxn = iter.next();
			TransactionUID tid = baseTxn.getTid();

			msg = " loadTransactions: processing local transaction " + tid;
			logger.log(Logger.DEBUG, msg);

			TransactionState state = baseTxn.getTransactionState();
			transactionList.addTransactionID(tid, state, false);
			TransactionWork txnWork = baseTxn.getTransactionWork();
			handleTransactionWork(tid, txnWork);
		}
	}

	public void loadClusterTransactions() throws BrokerException, IOException {

		List<BaseTransaction> incompleteTxns = store
				.getIncompleteTransactions(BaseTransaction.CLUSTER_TRANSACTION_TYPE);
		String msg = " loading " + incompleteTxns.size()
				+ " incomplete cluster transactions:  ";
		logger.log(Logger.DEBUG, msg);
		Iterator<BaseTransaction> iter = incompleteTxns.iterator();
		while (iter.hasNext()) {
			ClusterTransaction clusterTxn = (ClusterTransaction) iter.next();
			TransactionUID tid = clusterTxn.getTid();

			msg = " loadTransactions: processing cluster transaction " + tid;
			logger.log(Logger.DEBUG, msg);

			TransactionState state = clusterTxn.getTransactionState();
			transactionList.addTransactionID(tid, state, false);

			TransactionBroker[] transactionBrokers = clusterTxn
					.getTransactionBrokers();
			transactionList.logClusterTransaction(tid, state,
					transactionBrokers, true, false);
			TransactionWork txnWork = clusterTxn.getTransactionWork();
			handleTransactionWork(tid, txnWork);
		}
	}
	
	public void loadRemoteTransactions() throws BrokerException, IOException {

		List<BaseTransaction> incompleteTxns = store
				.getIncompleteTransactions(BaseTransaction.REMOTE_TRANSACTION_TYPE);
		String msg = " loading " + incompleteTxns.size()
				+ " incomplete remote transactions:  ";
		logger.log(Logger.DEBUG, msg);
		Iterator<BaseTransaction> iter = incompleteTxns.iterator();
		while (iter.hasNext()) {
			RemoteTransaction remoteTxn = (RemoteTransaction)iter.next();
			TransactionUID tid = remoteTxn.getTid();

			msg = " loadTransactions: processing remote transaction " + tid;
			logger.log(Logger.DEBUG, msg);
			TransactionState state = remoteTxn.getTransactionState();
			TransactionAcknowledgement  ta[] = remoteTxn.getTxnAcks();
			BrokerAddress remoteTransactionHomeBroker = remoteTxn.getTxnHomeBroker();
			
			transactionList.logRemoteTransaction(tid, state, ta, 
					remoteTransactionHomeBroker, true, true, false);			              									
		}
	}

	void handleTransactionWork(TransactionUID tid, TransactionWork txnWork)
			throws BrokerException {
		
		
		Destination.loadDestinations();
		Iterator itr = Destination.getAllDestinations();
        while (itr.hasNext()) {
            Destination d = (Destination)itr.next();
            d.load();
        }
		
		for (int i = 0; i < txnWork.numSentMessages(); i++) {

			TransactionWorkMessage msg = txnWork.getSentMessages().get(i);
			Packet packet = msg.getMessage();

			DestinationUID duid = msg.getDestUID();
			PacketReference pr = PacketReference.createReference(packet, duid,
					null);
			Destination d = Destination.getDestination(duid);
			logger.log(Logger.DEBUG,
					" loadTransactions: processing prepared sent message "
							+ packet.getMessageID());

			// queue message
			boolean result = d.queueMessage(pr, true);

			// store (should not really be persisted as we are using txnLog)
			// pr.store();

			// add message to transaction
			transactionList.addMessage(pr.getTransactionID(), pr
					.getSysMessageID(), true);

		}
		for (int i = 0; i < txnWork.numMessageAcknowledgments(); i++)

		{
			TransactionWorkMessageAck msgAck = txnWork
					.getMessageAcknowledgments().get(i);
			
			SysMessageID ackedSysMsgID = msgAck.getSysMessageID();
			logger.log(Logger.DEBUG,
					" loadTransactions: processing prepared acknowledged message "
							+ ackedSysMsgID);

			transactionList
					.addAcknowledgement(tid, ackedSysMsgID, msgAck
							.getConsumerID(), msgAck.getConsumerID(), true,
							false);
			transactionList.addOrphanAck(tid, ackedSysMsgID, msgAck
					.getConsumerID(), msgAck.getConsumerID());
			

			logger.log(Logger.DEBUG,
					" trying to unroute prepared acknowledged message "
							+ ackedSysMsgID);
			// need to unroute messages that have been consumed in a prepared transaction.
			// they cannot be redelivered unless the transaction rols back.
			PacketReference ackedMessage =Destination.get(ackedSysMsgID);
			Destination dest = ackedMessage.getDestination();
			dest.unrouteLoadedTransactionAckMessage(ackedMessage, msgAck.getConsumerID());

		}
	}

}
